// file: MutableMapC.java
// author: CS102 S14 Class
// date: March 20, 2014
//
// This is an implementation of a mutable map API. Unlike the Maps
// in java.util and in SW, the get function for this map returns
// an Option type to deal with the case where a key was not in the
// map.
//
public class MutableMapC<Key, Value> implements MutableMap<Key, Value> {
  
    // We'll have a list of nodes holding the key/value pairs.
    //
    private Node front;
    private int N;
    
    public MutableMapC() { 
        this.front = null; 
        this.N = 0;
    }
    
    private class Node {
        private Node next;
        private Key key;
        private Value value;     

        private Node(Key key, Value value, Node next) {
            this.key = key;
            this.value = value;
            this.next = next;
        }
        private Node getNext()   { return this.next; }
        private Key getKey()     { return this.key; }
        private Value getValue() { return this.value; }
        
        private void setNext(Node node) { this.next = node; }
        
        private Key lookAhead() { 
            if(this.getNext() != null)
                return this.getNext().getKey();
            else
                return null;
        }
    }
    
    public void put(Key key, Value value) {    
        Node newNode = new Node(key, value, front);
        this.front = newNode;
        this.N++;
    }
    
    public Option<Value> get(Key key) {    
        Node current = front;
        while(current != null) {
            if(current.getKey().equals(key))
                return new Some(current.getValue());
            current = current.getNext();
        }
        return new None();
    }
    
    public int size() { return this.N; }
    public boolean isEmpty() { return this.size() == 0; }
    
    public String toString() { 
        StringBuilder sb = new StringBuilder();
        Node current = this.front;
        while(current != null) {
            String 
                ks = current.getKey().toString(),
                vs = current.getValue().toString();
            sb.append(ks + "=" + vs);
            current = current.getNext();
            if(current != null) sb.append(", ");
        }
        return "[" + sb.toString() + "]"; }
    
    public void remove(Key key) { 
        // Remove all occurrences of the key from the front of the
        // list.
        //
        while(!this.isEmpty() && this.front.getKey().equals(key)) {
            this.front = this.front.getNext();
            this.N--;
        }
        if(this.isEmpty()) return;
        
        Node current = this.front;
        while(current.getNext() != null) {
            Node next = current.getNext();
            if (next.getKey().equals(key)) {
                current.setNext(next.getNext());
                this.N--;
            }
            else 
                current = next;
        }
    }

    public static void main(String[] args) {
     
        MutableMap<String, String> myMap = new MutableMapC<String, String>();
        
        myMap.put("A", "Tom");
        myMap.put("B", "Bill");
        myMap.put("A", "Alice");
        myMap.put("A", "Mary");
        myMap.put("J", "Joe");
        System.out.println("map = " + myMap.toString());
        myMap.remove("A");
        System.out.println("B = " + myMap.get("B").valueOf());
        System.out.println("map = " + myMap.toString());
    }
}